# Zach Gildersleeve
# zgildersleeve@gmail.com
# CS6630 - Scientific Visualization
# Project 5
# December 8, 2006
#
# vectorCalcVis.tcl - This script draws streamlines from each critical point
# as in vectorVis.tcl, and also draws the isosurface colormapped with either
# the divergence or curl magnitude values.

catch {load vtktcl}
set VTK_TCL ".."
set VTK_DATA "../vtkdata"

set volume $argv

##########
# Read in magnitude data, and make a contoured isosurface.
# Colors the isosurface and creates an actor.
#
vtkStructuredPointsReader reader
	reader SetFileName $VTK_DATA/ChalVec$volume.vtk
	reader Update

vtkStructuredPointsReader reader_magnitude
	reader_magnitude SetFileName $VTK_DATA/ChalMag$volume.vtk
	reader_magnitude Update

	set mags [reader_magnitude GetOutput]
	set bounds [$mags GetBounds]
	set x0 [lindex $bounds 0]
	set x1 [lindex $bounds 1]
	set y0 [lindex $bounds 2]
	set y1 [lindex $bounds 3]
	set z0 [lindex $bounds 4]
	set z1 [lindex $bounds 5]
	puts "VOLUME SIZE $x0 $x1 $y0 $y1 $z0 $z1"

	set range [$mags GetScalarRange]
	set v0 [lindex $range 0]
	set v1 [lindex $range 1]
	puts "DATA RANGE: $v0 $v1"

vtkStructuredPointsReader reader_curl
	reader_curl SetFileName $VTK_DATA/ChalCMa$volume.vtk
	reader_curl Update

	set mags [reader_curl GetOutput]
	set range [$mags GetScalarRange]
	set cv0 [lindex $range 0]
	set cv1 [lindex $range 1]
	puts "CURL MAGNITUDE DATA RANGE: $cv0 $cv1"

vtkStructuredPointsReader reader_divergence
	reader_divergence SetFileName $VTK_DATA/ChalDMag$volume.vtk
	reader_divergence Update

	set mags [reader_divergence GetOutput]
	set range [$mags GetScalarRange]
	set dv0 [lindex $range 0]
	set dv1 [lindex $range 1]
	puts "DIVERGENCE DATA RANGE: $dv0 $dv1"

##########
# Build the lookup table
#
vtkLookupTable lut_magnitude
	lut_magnitude SetTableRange $v0 $v1
	lut_magnitude SetHueRange 0.0 0.7
	lut_magnitude SetSaturationRange 1 1
	lut_magnitude SetAlphaRange 0.4 0.7
	lut_magnitude Build

vtkLookupTable lut_divergence
	lut_divergence SetTableRange $dv0 $dv1
	lut_divergence SetHueRange 0.0 0.7
	lut_divergence SetSaturationRange 1 1
	lut_divergence SetAlphaRange 0.2 0.3
	lut_divergence Build

vtkLookupTable lut_curl
	lut_curl SetTableRange $cv0 $cv1
	lut_curl SetHueRange 0.0 0.7
	lut_curl SetSaturationRange 1 1
	lut_curl SetAlphaRange 0.2 0.3
	lut_curl Build


##########
# Initializes multiple copies of each component.  Must be done here
# rather than in setMultipleSphere() to avoid duplicate initialization
#
for {set i 0} {$i < 6} {incr i} {
	vtkSphereSource sphere$i
	vtkStreamLine stream$i
	vtkRuledSurfaceFilter scalarSurface$i
	vtkTubeFilter tube$i
	vtkPolyDataMapper dataMapper$i
	vtkActor dataActor$i

	vtkProbeFilter probe$i
	vtkMaskPoints mask$i
	vtkGlyph3D glyph$i
	vtkPolyDataMapper glyphMapper$i
	vtkActor glyphActor$i
}

vtkRungeKutta4 integ

vtkConeSource cone
	cone SetResolution 8
	cone SetHeight	1.0
	cone SetRadius	0.2

vtkTransform transform
	transform Translate 0.0 0.0 0.0

vtkTransformPolyDataFilter transformFilter
	transformFilter SetInput [cone GetOutput]
	transformFilter SetTransform transform

##########
# Set the located critical points for the two challenge volumes
#
vtkPoints challengePoints0
	challengePoints0 InsertNextPoint 38.8 26.1 32.1
	challengePoints0 InsertNextPoint 17.0 24.5 13.9
	challengePoints0 InsertNextPoint 25.0 15.0 18.8
	challengePoints0 InsertNextPoint 16.1 16.8 25.0

vtkPoints challengePoints1
	challengePoints1 InsertNextPoint 35.0 15.0 20.0
	challengePoints1 InsertNextPoint 54.9 15.2 20.0
	challengePoints1 InsertNextPoint 65.0 45.0 20.0
	challengePoints1 InsertNextPoint 45.0 45.0 20.0
	challengePoints1 InsertNextPoint 20.0 30.0 20.0
	challengePoints1 InsertNextPoint  9.0 32.0 16.0

###
# Builds the streamline, glyph, and critical point pipeline for the two
# challenge volumes.  Pulls point data from challengePoints.  Pipeline steps
# are detailed below in main program.
#
proc setMultipleSphere {args} {

	global v0
	global v1
	global volume

	for {set i 0} {$i < 6} {incr i} {

		#vtkSphereSource sphere$i
 			sphere$i SetRadius 2
  			sphere$i SetCenter 	[lindex [challengePoints$volume GetPoint $i] 0] \
						[lindex [challengePoints$volume GetPoint $i] 1] \
						[lindex [challengePoints$volume GetPoint $i] 2]
  			sphere$i SetThetaResolution 1


		# draws a streamline for each point on the sphere
		#vtkStreamLine stream$i
			stream$i SetInput [reader GetOutput]
			stream$i SetSource [sphere$i GetOutput]
			stream$i SetIntegrator integ
			stream$i SetMaximumPropagationTime 500
			stream$i SetIntegrationStepLength 0.1
			stream$i SetIntegrationDirectionToIntegrateBothDirections
			stream$i SetStepLength 0.1

		#vtkRuledSurfaceFilter scalarSurface$i
			scalarSurface$i SetInput [stream$i GetOutput]
			scalarSurface$i SetOffset 0
			scalarSurface$i SetOnRatio 2
			scalarSurface$i PassLinesOn
			scalarSurface$i SetRuledModeToPointWalk
			scalarSurface$i SetDistanceFactor 50

		#vtkTubeFilter tube$i
			tube$i SetInput [scalarSurface$i GetOutput]
			tube$i SetRadius 0.1
			tube$i SetNumberOfSides 6

		#vtkPolyDataMapper dataMapper$i
			dataMapper$i SetInput [tube$i GetOutput]
			eval dataMapper$i SetScalarRange $v0 $v1
			dataMapper$i SetLookupTable lut_magnitude

		#vtkActor dataActor$i
			dataActor$i SetMapper dataMapper$i

		ren1 AddActor dataActor$i

		#vtkProbeFilter probe$i
			probe$i SetInput [stream$i GetOutput]
			probe$i SetSource [reader GetOutput]

		#vtkMaskPoints mask$i
			mask$i SetInput [probe$i GetOutput]
			mask$i SetOnRatio 60
			mask$i RandomModeOn

		#vtkGlyph3D glyph$i
			glyph$i SetInput [mask$i GetOutput]
			glyph$i SetSource [transformFilter GetOutput]
			glyph$i SetScaleModeToScaleByVector
			glyph$i SetScaleFactor 2
			glyph$i SetVectorModeToUseVector
			glyph$i SetColorModeToColorByVector

		#vtkPolyDataMapper glyphMapper$i
			glyphMapper$i SetInput [glyph$i GetOutput]
			glyphMapper$i SetLookupTable lut_magnitude

		#vtkActor glyphActor$i
			glyphActor$i SetMapper glyphMapper$i

		ren1 AddActor glyphActor$i

	}
}

vtkContourFilter contour
	contour SetInput [reader_magnitude GetOutput]
	contour SetNumberOfContours 1
	contour SetValue 0 0.5

vtkPolyDataNormals normals
	normals SetInput [contour GetOutput]
	normals SetFeatureAngle 60
	normals ConsistencyOff
	normals SplittingOff

vtkProbeFilter probe
	probe SetInput [normals GetOutput]
	probe SetSource [reader_curl GetOutput]

vtkPolyDataMapper mapper_magnitude
	mapper_magnitude SetInput [probe GetOutput]
	mapper_magnitude SetLookupTable lut_divergence
	mapper_magnitude SetColorModeToMapScalars
	eval mapper_magnitude SetScalarRange $dv0 $dv1

vtkActor actor_magnitude
	actor_magnitude SetMapper mapper_magnitude

##########
# Draws the XYZ colored positional lines on the bounding box.
# Red	= X axis
# Green = Y axis
# Blue	= Z axis
#
vtkLineSource line
	line SetPoint1 0 0 0
	line SetPoint2 10 0 0

vtkTubeFilter fatline
	fatline SetInput [line GetOutput]
	fatline SetRadius 0.15
	fatline SetNumberOfSides 30

vtkPolyDataMapper lineMapper
	lineMapper SetInput [fatline GetOutput]

vtkActor lineXActor
	lineXActor SetMapper lineMapper
	[lineXActor GetProperty] SetColor 1 0.2 0.2

vtkActor lineYActor
	lineYActor SetMapper lineMapper
	lineYActor RotateZ 90
	[lineYActor GetProperty] SetColor 0.2 1 0.2

vtkActor lineZActor
	lineZActor SetMapper lineMapper
	lineZActor RotateY -90
	[lineZActor GetProperty] SetColor 0.2 0.2 1

##########
# Create the scalar colormap bar actor
#
vtkScalarBarActor bar
	bar SetLookupTable lut_divergence

##########
# Creates volume bounding box and actor
#
vtkOutlineFilter outline
	outline	SetInput [reader_magnitude GetOutput]

vtkPolyDataMapper outlineMapper
	outlineMapper SetInput [outline GetOutput]

vtkActor outlineActor
	outlineActor SetMapper outlineMapper
	eval [outlineActor GetProperty] SetColor 1 1 1

##########
# Create the Renderer and Windows
#
vtkRenderer ren1

setMultipleSphere
	
vtkRenderWindow renWin
	renWin AddRenderer ren1
	renWin SetSize 512 512
vtkRenderWindowInteractor iren
	iren SetRenderWindow renWin
	ren1 AddActor actor_magnitude
	ren1 AddActor outlineActor
	ren1 AddActor bar
	ren1 AddActor lineXActor
	ren1 AddActor lineYActor
	ren1 AddActor lineZActor

	ren1 SetBackground 0.5 0.5 0.5
	ren1 ResetCamera

vtkInteractorStyleTrackballCamera style
	iren SetInteractorStyle style

renWin Render

#########
# Set up the GUI widgets
#
frame .f1
frame .f1.keepMe
button .f1.keepMe.quit -text "Quit" -command {exit}
button .f1.keepMe.reset -text "Reset Camera" -command resetCamera

frame .f2
frame .f2.contour
label .f2.contour.space -text " "
label .f2.contour.l1 -text "Select Contour Value"
scale .f2.contour.s1 -from $v0 -to $v1 -orient horizontal -command setContour -length 200 -resolution 0.001 -showvalue true
radiobutton .f2.contour.r2 -text "Color with Divergence Field" -command setData -var dataValue -value divergence
radiobutton .f2.contour.r3 -text "Color with Curl Magnitude" -command setData -var dataValue -value curl

#########
# Set the defaults for the GUI
#
set dataValue divergence

##########
# Arrange the widgets and GUI elements in each
# respective frame
#
grid .f1.keepMe -row 0 -column 0
grid .f1.keepMe.quit -row 1 -column 0
grid .f1.keepMe.reset -row 2 -column 0

grid .f2.contour -row 0 -column 0
grid .f2.contour.l1 -row 1 -column 0
grid .f2.contour.s1 -row 1 -column 1
grid .f2.contour.r2 -row 10 -column 0
grid .f2.contour.r3 -row 11 -column 0

##########
# Arrange the frames in the GUI window
#
grid .f1 -row 0 -sticky ew
grid .f2 -row 1 -sticky ew

###
# Resets the camera. 
#
proc resetCamera {args} {
	ren1 ResetCamera
	ren1 ResetCameraClippingRange
	[ren1 GetActiveCamera] OrthogonalizeViewUp
	[ren1 GetActiveCamera] SetViewUp 0 1 0
	renWin Render
}

###
#
#
proc setData {args} {
	global dataValue
	global volume
	global cv0 cv1 dv0 dv1

	switch $dataValue {
		"divergence"	{
				eval mapper_magnitude SetScalarRange $dv0 $dv1
				probe SetSource [reader_divergence GetOutput]
				mapper_magnitude SetLookupTable lut_divergence
				bar SetLookupTable lut_divergence
		}
		"curl"		{
				eval mapper_magnitude SetScalarRange $cv0 $cv1
				probe SetSource [reader_curl GetOutput]
				mapper_magnitude SetLookupTable lut_curl
				bar SetLookupTable lut_curl
		}
	}

	renWin Render
}

###
#
#
proc setContour {args} {
	contour SetValue 0 [.f2.contour.s1 get]
	
	renWin Render
}
