# Zach Gildersleeve
# zgildersleeve@gmail.com
# CS6630 - Scientific Visualization
# Project 5
# December 8, 2006
#
# vectorVis.tcl - This script, based on the vtk examples StreamSurface.tcl
# and spikeF.tcl, reads in vector volume data and creates streamlines based on
# spherical points.  Cone glyphs are colored accordingly, and added random across
# the streamlines.  Using a magnitude volume data, it is possible to
# manually identify critical points in the volume data.

catch {load vtktcl}
set VTK_TCL ".."
set VTK_DATA "../vtkdata"
set howManyActors 1

##########
# Build the lookup table
#
vtkLookupTable HSVlut
	HSVlut SetTableRange 0.0 1.0
	HSVlut SetHueRange 0.0 0.7
	HSVlut SetSaturationRange 1 1
	HSVlut SetAlphaRange 0.3 0.5
	HSVlut Build

##########
# Set the located critical points for the two challenge volumes
#
vtkPoints challengePoints1
	challengePoints1 InsertNextPoint 38.8 26.1 32.1
	challengePoints1 InsertNextPoint 17.0 24.5 13.9
	challengePoints1 InsertNextPoint 25.0 15.0 18.8
	challengePoints1 InsertNextPoint 16.1 16.8 25.0

vtkPoints challengePoints2
	challengePoints2 InsertNextPoint 35.0 15.0 20.0
	challengePoints2 InsertNextPoint 54.9 15.2 20.0
	challengePoints2 InsertNextPoint 65.0 45.0 20.0
	challengePoints2 InsertNextPoint 45.0 45.0 20.0
	challengePoints2 InsertNextPoint 20.0 30.0 20.0
	challengePoints2 InsertNextPoint  9.0 32.0 16.0

##########
# Read in vector data and load boundary conditions.
#
vtkStructuredPointsReader reader
	reader SetFileName $VTK_DATA/TestVec0.vtk
	reader Update

	set mags [reader GetOutput]
	set bounds [$mags GetBounds]
	set x0 [lindex $bounds 0]
	set x1 [lindex $bounds 1]
	set y0 [lindex $bounds 2]
	set y1 [lindex $bounds 3]
	set z0 [lindex $bounds 4]
	set z1 [lindex $bounds 5]
	puts "VOLUME SIZE $x0 $x1 $y0 $y1 $z0 $z1"

	set range [$mags GetScalarRange]
	set v0 [lindex $range 0]
	set v1 [lindex $range 1]
	puts "DATA RANGE: $v0 $v1"

##########
# Read in magnitude data, and make a contoured isosurface.
# Colors the isosurface and creates an actor.
#
	vtkStructuredPointsReader reader_magnitude
		reader_magnitude SetFileName $VTK_DATA/TestMag0.vtk

	vtkContourFilter contour
		contour SetInput [reader_magnitude GetOutput]
		contour SetValue 0 1

	vtkPolyDataNormals normals
		normals SetInput [contour GetOutput]
		normals SetFeatureAngle 60
		normals ConsistencyOff
		normals SplittingOff

	vtkPolyDataMapper mapper_magnitude
		mapper_magnitude SetInput [normals GetOutput]
		mapper_magnitude SetLookupTable HSVlut
		mapper_magnitude SetColorModeToMapScalars
		eval mapper_magnitude SetScalarRange $v0 $v1

	vtkActor actor_magnitude
		actor_magnitude SetMapper mapper_magnitude

##########
# Initializes multiple copies of each component.  Must be done here
# rather than in setMultipleSphere() to avoid duplicate initialization
#
for {set i 0} {$i < 6} {incr i} {

	vtkSphereSource sphere$i
	vtkStreamLine stream$i
	vtkRuledSurfaceFilter scalarSurface$i
	vtkTubeFilter tube$i
	vtkPolyDataMapper dataMapper$i
	vtkActor dataActor$i

	vtkSphereSource criticalMarker$i
	vtkPolyDataMapper criticalMapper$i
	vtkActor criticalActor$i

	vtkProbeFilter probe$i
	vtkMaskPoints mask$i
	vtkGlyph3D glyph$i
	vtkPolyDataMapper glyphMapper$i
	vtkActor glyphActor$i

	vtkPlaneSource plane$i
}

###
# Builds the streamline, glyph, and critical point pipeline for the two
# challenge volumes.  Pulls point data from challengePoints.  Pipeline steps
# are detailed below in main program.
#
proc setMultipleSphere {args} {

	global v0
	global v1
	global showPointsCheck
	global howManyActors

	if {$args == 1} {	set howManyActors 4
	} elseif {$args == 2} {	set howManyActors 6
	} else {puts "Invalid Parameters in proc setMultipleSphere"}

	for {set i 0} {$i < $howManyActors} {incr i} {

		#vtkSphereSource sphere$i
 			sphere$i SetRadius 2
  			sphere$i SetCenter 	[lindex [challengePoints$args GetPoint $i] 0] \
						[lindex [challengePoints$args GetPoint $i] 1] \
						[lindex [challengePoints$args GetPoint $i] 2]
  			sphere$i SetThetaResolution 1

		# draws a streamline for each point on the sphere
		#vtkStreamLine stream$i
			stream$i SetInput [reader GetOutput]
			stream$i SetSource [sphere$i GetOutput]
			stream$i SetIntegrator integ
			stream$i SetMaximumPropagationTime 500
			stream$i SetIntegrationStepLength 0.1
			stream$i SetIntegrationDirectionToIntegrateBothDirections
			stream$i SetStepLength 0.1

		#vtkRuledSurfaceFilter scalarSurface$i
			scalarSurface$i SetInput [stream$i GetOutput]
			scalarSurface$i SetOffset 0
			scalarSurface$i SetOnRatio 2
			scalarSurface$i PassLinesOn
			scalarSurface$i SetRuledModeToPointWalk
			scalarSurface$i SetDistanceFactor 50

		#vtkTubeFilter tube$i
			tube$i SetInput [scalarSurface$i GetOutput]
			tube$i SetRadius 0.1
			tube$i SetNumberOfSides 6

		#vtkPolyDataMapper dataMapper$i
			dataMapper$i SetInput [tube$i GetOutput]
			eval dataMapper$i SetScalarRange $v0 $v1
			dataMapper$i SetLookupTable HSVlut

		#vtkActor dataActor$i
			dataActor$i SetMapper dataMapper$i

		ren1 AddActor dataActor$i

		#vtkSphereSource criticalMarker$i
  			criticalMarker$i SetRadius 1.0
  			criticalMarker$i SetCenter 	[lindex [challengePoints$args GetPoint $i] 0] \
							[lindex [challengePoints$args GetPoint $i] 1] \
							[lindex [challengePoints$args GetPoint $i] 2]
  			criticalMarker$i SetThetaResolution 10

		#vtkPolyDataMapper criticalMapper$i
			criticalMapper$i SetInput [criticalMarker$i GetOutput]

		#vtkActor criticalActor$i
			criticalActor$i SetMapper criticalMapper$i
			[criticalActor$i GetProperty] SetColor 1.0 0.0 0.0
			[criticalActor$i GetProperty] SetOpacity 0.5

		if {$showPointsCheck} {ren1 AddActor criticalActor$i}

		#vtkProbeFilter probe$i
			probe$i SetInput [stream$i GetOutput]
			probe$i SetSource [reader GetOutput]

		#vtkMaskPoints mask$i
			mask$i SetInput [probe$i GetOutput]
			mask$i SetOnRatio 60
			mask$i RandomModeOn

		#vtkGlyph3D glyph$i
			glyph$i SetInput [mask$i GetOutput]
			glyph$i SetSource [transformFilter GetOutput]
			glyph$i SetScaleModeToScaleByVector
			glyph$i SetScaleFactor 1
			glyph$i SetVectorModeToUseVector
			glyph$i SetColorModeToColorByVector

		#vtkPolyDataMapper glyphMapper$i
			glyphMapper$i SetInput [glyph$i GetOutput]
			glyphMapper$i SetLookupTable HSVlut

		#vtkActor glyphActor$i
			glyphActor$i SetMapper glyphMapper$i

		ren1 AddActor glyphActor$i

	}
	renWin Render
}

##########
# Generates streamline starting points from a sphere source, builds a streamline from
# each starting point, and connects them together as a ruled surface.
# Creates an actor for the streamlines
#
vtkSphereSource sphere
	sphere SetRadius 2
	sphere SetCenter 15 15 15
	sphere SetThetaResolution 10


vtkRungeKutta4 integ

# draws a streamline for each point on the sphere
vtkStreamLine stream
	stream SetInput [reader GetOutput]
	stream SetSource [sphere GetOutput]
	stream SetIntegrator integ
	stream SetMaximumPropagationTime 500
	stream SetIntegrationStepLength 0.1
	stream SetIntegrationDirectionToIntegrateBothDirections
	stream SetStepLength 0.1

vtkRuledSurfaceFilter scalarSurface
	scalarSurface SetInput [stream GetOutput]
	scalarSurface SetOffset 0
	scalarSurface SetOnRatio 2
	scalarSurface PassLinesOn
	scalarSurface SetRuledModeToPointWalk
	scalarSurface SetDistanceFactor 50

vtkTubeFilter tube
	tube SetInput [scalarSurface GetOutput]
	tube SetRadius 0.1
	tube SetNumberOfSides 6

vtkPolyDataMapper dataMapper
	dataMapper SetInput [tube GetOutput]
	eval dataMapper SetScalarRange $v0 $v1
	dataMapper SetLookupTable HSVlut

vtkActor dataActor
	dataActor SetMapper dataMapper

##########
# Sets a cone glyph along each streamline, and builds
# a separate glyph actor.  Colors and scales each cone
# accordingly.
#
vtkProbeFilter probe
	probe SetInput [stream GetOutput]
	probe SetSource [reader GetOutput]

vtkMaskPoints mask
	mask SetInput [probe GetOutput]
	mask SetOnRatio 60
	mask RandomModeOn

vtkConeSource cone
	cone SetResolution 8
	cone SetHeight	1.0
	cone SetRadius	0.2

vtkTransform transform
	transform Translate 0.0 0.0 0.0

vtkTransformPolyDataFilter transformFilter
	transformFilter SetInput [cone GetOutput]
	transformFilter SetTransform transform

vtkGlyph3D glyph
	glyph SetInput [mask GetOutput]
	glyph SetSource [transformFilter GetOutput]
	glyph SetScaleModeToScaleByVector
	glyph SetScaleFactor 1
	glyph SetVectorModeToUseVector
	glyph SetColorModeToColorByVector

vtkPolyDataMapper glyphMapper
	glyphMapper SetInput [glyph GetOutput]
	glyphMapper SetLookupTable HSVlut

vtkActor glyphActor
	glyphActor SetMapper glyphMapper

##########
# Creates volume bounding box and actor
#
vtkOutlineFilter outline
	outline	SetInput [reader GetOutput]

vtkPolyDataMapper outlineMapper
	outlineMapper SetInput [outline GetOutput]

vtkActor outlineActor
	outlineActor SetMapper outlineMapper
	eval [outlineActor GetProperty] SetColor 1 1 1

#########
# Creates a transparent red sphere to mark each critical point with
#
vtkSphereSource criticalMarker
	criticalMarker SetRadius 1.0
	criticalMarker SetCenter 15 15 15
	criticalMarker SetThetaResolution 10

vtkPolyDataMapper criticalMapper
	criticalMapper SetInput [criticalMarker GetOutput]

vtkActor criticalActor
	criticalActor SetMapper criticalMapper
	[criticalActor GetProperty] SetColor 1.0 0.0 0.0
	[criticalActor GetProperty] SetOpacity 0.5

##########
# Draws the XYZ colored positional lines on the bounding box.
# Red	= X axis
# Green = Y axis
# Blue	= Z axis
#
vtkLineSource line
	line SetPoint1 0 0 0
	line SetPoint2 [expr $x1 / 3.0] 0 0
vtkTubeFilter fatline
	fatline SetInput [line GetOutput]
	fatline SetRadius 0.15
	fatline SetNumberOfSides 30
vtkPolyDataMapper lineMapper
	lineMapper SetInput [fatline GetOutput]

vtkActor lineXActor
	lineXActor SetMapper lineMapper
	[lineXActor GetProperty] SetColor 1 0.2 0.2
vtkActor lineYActor
	lineYActor SetMapper lineMapper
	lineYActor RotateZ 90
	[lineYActor GetProperty] SetColor 0.2 1 0.2
vtkActor lineZActor
	lineZActor SetMapper lineMapper
	lineZActor RotateY -90
	[lineZActor GetProperty] SetColor 0.2 0.2 1

##########
# Create the scalar colormap bar actor
#
vtkScalarBarActor bar
	bar SetLookupTable HSVlut

##########
# Create the Renderer and Windows
#
vtkRenderer ren1
	
vtkRenderWindow renWin
	renWin AddRenderer ren1
	renWin SetSize 512 512
vtkRenderWindowInteractor iren
	iren SetRenderWindow renWin
	ren1 AddActor outlineActor
	ren1 AddActor dataActor
	ren1 AddActor glyphActor
	ren1 AddActor bar
	ren1 AddActor lineXActor
	ren1 AddActor lineYActor
	ren1 AddActor lineZActor

	ren1 SetBackground 0.5 0.5 0.5
	ren1 ResetCamera

vtkInteractorStyleTrackballCamera style
	iren SetInteractorStyle style

renWin Render

#########
# Set up the GUI widgets
#
frame .f1
frame .f1.keepMe
button .f1.keepMe.quit -text "Quit" -command {exit}
button .f1.keepMe.reset -text "Reset Camera" -command resetCamera

frame .f2
frame .f2.data
radiobutton .f2.data.r1 -text "Test Volume 1" -command setData -var dataValue -value test1 -anchor w
radiobutton .f2.data.r2 -text "Test Volume 2" -command setData -var dataValue -value test2 -anchor w
radiobutton .f2.data.r3 -text "Test Volume 3" -command setData -var dataValue -value test3 -anchor w
radiobutton .f2.data.r4 -text "Challenge Volume 1" -command setData -var dataValue -value challenge1 -anchor w
radiobutton .f2.data.r5 -text "Challenge Volume 2" -command setData -var dataValue -value challenge2 -anchor w
checkbutton .f2.data.c1 -text "Manual Critical Points" -command setManualPoints -var manualPointsCheck -anchor w
checkbutton .f2.data.c2 -text "Show Critical Points" -command setManualPoints -var showPointsCheck -anchor w
checkbutton .f2.data.c3 -text "Large Scale Visualization" -command setLargeScale -var largeScaleCheck -anchor w
label .f2.data.space -text " "
scale .f2.data.s1 -from 0 -to 50 -orient horizontal -command setParam -length 100 -resolution 1 -showvalue true
label .f2.data.l1 -text "Number of Streamlines"

frame .f3
frame .f3.surface
radiobutton .f3.surface.r1 -text "Vectors" -command setSurface -var surfaceValue -value vector
radiobutton .f3.surface.r2 -text "Magnitude" -command setSurface -var surfaceValue -value magnitude
radiobutton .f3.surface.r3 -text "Both" -command setSurface -var surfaceValue -value both

frame .f4
frame .f4.contour
scale .f4.contour.s1 -from 0 -to 10 -orient horizontal -command setContour -length 100 -resolution 0.01 -showvalue true
label .f4.contour.l1 -text "Contour Value"

frame .f5
frame .f5.critical
scale .f5.critical.s1 -from 0 -to 80 -orient horizontal -command setCritical -length 100 -resolution 0.1 -showvalue true
label .f5.critical.l1 -text "X Position"
scale .f5.critical.s2 -from 0 -to 60 -orient horizontal -command setCritical -length 100 -resolution 0.1 -showvalue true
label .f5.critical.l2 -text "Y Position"
scale .f5.critical.s3 -from 0 -to 40 -orient horizontal -command setCritical -length 100 -resolution 0.1 -showvalue true
label .f5.critical.l3 -text "Z Position"

#########
# Set the defaults for the GUI
#
set dataValue test1
.f2.data.s1 set 10
set surfaceValue vector
set manualPointsCheck 0
set showPointsCheck 0

##########
# Arrange the widgets and GUI elements in each
# respective frame
#
grid .f1.keepMe -row 0 -column 0
grid .f1.keepMe.quit -row 1 -column 0
grid .f1.keepMe.reset -row 2 -column 0

grid .f2.data -row 0 -column 0
grid .f2.data.r1 -row 1 -column 0 -sticky ew
grid .f2.data.r2 -row 2 -column 0 -sticky ew
grid .f2.data.r3 -row 3 -column 0 -sticky ew
grid .f2.data.r4 -row 4 -column 0 -sticky ew
grid .f2.data.r5 -row 5 -column 0 -sticky ew
grid .f2.data.space -row 6 -column 0
grid .f2.data.c1 -row 7 -column 0 -sticky ew
grid .f2.data.c2 -row 8 -column 0 -sticky ew
grid .f2.data.c3 -row 9 -column 0 -sticky ew
grid .f2.data.s1 -row 10 -column 1
grid .f2.data.l1 -row 10 -column 0

grid .f3.surface -row 0 -column 0
grid .f3.surface.r1 -row 1 -column 0 -sticky ew
grid .f3.surface.r2 -row 2 -column 0 -sticky ew
grid .f3.surface.r3 -row 3 -column 0 -sticky ew

grid .f4.contour -row 0 -column 0
grid .f4.contour.s1 -row 1 -column 1
grid .f4.contour.l1 -row 1 -column 0

grid .f5.critical -row 0 -column 0
grid .f5.critical.s1 -row 1 -column 1
grid .f5.critical.l1 -row 1 -column 0
grid .f5.critical.s2 -row 2 -column 1
grid .f5.critical.l2 -row 2 -column 0
grid .f5.critical.s3 -row 3 -column 1
grid .f5.critical.l3 -row 3 -column 0

##########
# Arrange the frames in the GUI window
#
grid .f1 -row 0 -sticky ew
grid .f2 -row 1 -sticky ew
grid .f3 -row 2 -sticky ew
grid .f4 -row 3 -sticky ew

###
# Removes all streamlines, critical points, and glyph actors
#
proc removeActors {args} {

	ren1 RemoveActor dataActor
	ren1 RemoveActor criticalActor
	ren1 RemoveActor glyphActor

	for {set i 0} {$i < 6} {incr i} {
		ren1 RemoveActor dataActor$i
		ren1 RemoveActor criticalActor$i
		ren1 RemoveActor glyphActor$i
	}
}

###
# Resets the camera. 
#
proc resetCamera {args} {
	ren1 ResetCamera
	ren1 ResetCameraClippingRange
	[ren1 GetActiveCamera] OrthogonalizeViewUp
	[ren1 GetActiveCamera] SetViewUp 0 1 0
	renWin Render
}

###
# Sets the input dataset.  First removes all components from the window,
# then associated the readers with the correct files.  The boundaries
# for the new dataset are returned, and the appropriate actors are added.
#
proc setData {args} {
	global dataValue
	global VTK_DATA "../vtkdata"
	global mags bounds x0 x1 y0 y1 z0 z1 range v0 v1


	ren1 Clear
	removeActors

	switch $dataValue {
		"test1"		{	reader SetFileName $VTK_DATA/TestVec0.vtk
					reader_magnitude SetFileName $VTK_DATA/TestMag0.vtk
					.f2.data.s1 set 10	
					ren1 AddActor dataActor
					ren1 AddActor glyphActor		
		}
		"test2"		{  	reader SetFileName $VTK_DATA/TestVec1.vtk
					reader_magnitude SetFileName $VTK_DATA/TestMag1.vtk
					.f2.data.s1 set 10
					ren1 AddActor dataActor
					ren1 AddActor glyphActor
		}
		"test3"		{  	reader SetFileName $VTK_DATA/TestVec2.vtk
					reader_magnitude SetFileName $VTK_DATA/TestMag2.vtk
					.f2.data.s1 set 10
					ren1 AddActor dataActor
					ren1 AddActor glyphActor
		}
		"challenge1"	{ 	reader SetFileName $VTK_DATA/ChalVec0.vtk
					reader_magnitude SetFileName $VTK_DATA/ChalMag0.vtk
					setMultipleSphere 1
					.f2.data.s1 set 1
		}
		"challenge2"	{ 	reader SetFileName $VTK_DATA/ChalVec1.vtk
					reader_magnitude SetFileName $VTK_DATA/ChalMag1.vtk
					setMultipleSphere 2
					.f2.data.s1 set 1
		}
	}

	set mags [reader GetOutput]
	set bounds [$mags GetBounds]
	set x0 [lindex $bounds 0]
	set x1 [lindex $bounds 1]
	set y0 [lindex $bounds 2]
	set y1 [lindex $bounds 3]
	set z0 [lindex $bounds 4]
	set z1 [lindex $bounds 5]
	puts "VOLUME SIZE $x0 $x1 $y0 $y1 $z0 $z1"

	set range [$mags GetScalarRange]
	set v0 [lindex $range 0]
	set v1 [lindex $range 1]
	puts "DATA RANGE: $v0 $v1"
	
	ren1 AddActor bar
	ren1 AddActor outlineActor
	set surfaceValue magnitude

	resetCamera
	renWin Render
}

###
# Sets the resolution by changing the resolution of the
# of the sphere generating the streamlines.  Less resolution
# means less sphere geometry means less generated points.
#
proc setParam {args} {
	sphere SetThetaResolution [.f2.data.s1 get]
	renWin Render
}

###
# Sets the current contour value for the isosurface
#
proc setContour {args} {
	contour SetValue 0 [.f4.contour.s1 get]
	renWin Render
}

###
# Sets whether the vector streamlines, the isosurface,
# or both, are drawn to the screen.
#
proc setSurface {args} {
	global surfaceValue

	ren1 Clear
	removeActors

	switch $surfaceValue {
		"magnitude" {	setData
		}
		"vector" {	ren1 AddActor dataActor
			  	ren1 AddActor glyphActor
		}
		"both"	{	setData
				ren1 AddActor actor_magnitude
		}
	}
	ren1 AddActor bar
	ren1 AddActor outlineActor
	renWin Render
}

###
#  Allows for manual setting of one critical point by controlling sliders.
#
proc setCritical {args} {
	global manualPointsCheck

	if [expr $manualPointsCheck] {

  	sphere SetCenter [.f5.critical.s1 get] [.f5.critical.s2 get] [.f5.critical.s3 get]
  	criticalMarker SetCenter [.f5.critical.s1 get] [.f5.critical.s2 get] [.f5.critical.s3 get]
	renWin Render
	}
}

###
# Adds or removes the sphere critical markers depending on the checkbox selection.
#
proc setManualPoints {args} {
	global manualPointsCheck
	global showPointsCheck
	global dataValue

	ren1 RemoveActor criticalActor
	ren1 RemoveActor criticalActor0	
	ren1 RemoveActor criticalActor1
	ren1 RemoveActor criticalActor2
	ren1 RemoveActor criticalActor3
	ren1 RemoveActor criticalActor4
	ren1 RemoveActor criticalActor5

	if {$manualPointsCheck == 1} {	if {$showPointsCheck == 1} {ren1 AddActor criticalActor}
					grid .f5 -row 4 -sticky ew
	} else {
		grid forget .f5
		if {$dataValue == "challenge1"} {	if {$showPointsCheck == 1} {	ren1 AddActor criticalActor0
											ren1 AddActor criticalActor1
											ren1 AddActor criticalActor2
											ren1 AddActor criticalActor3}

		} elseif {$dataValue == "challenge2"} {	if {$showPointsCheck == 1} {	ren1 AddActor criticalActor0
											ren1 AddActor criticalActor1
											ren1 AddActor criticalActor2
											ren1 AddActor criticalActor3
											ren1 AddActor criticalActor4
											ren1 AddActor criticalActor5}
		} else {	if {$showPointsCheck == 1} {ren1 AddActor criticalActor}
			  	sphere SetCenter 15 15 15}
	}
	renWin Render
}

###
# Uses bounding box as source for streamlines rather than critical points.
#
proc setLargeScale {args} {
	global largeScaleCheck
	global mags bounds x0 x1 y0 y1 z0 z1 range v0 v1
	
	if {$largeScaleCheck} {	ren1 Clear
				removeActors
				
				#left side
				plane0 SetOrigin $x0 $y0 $z0
				plane0 SetPoint1 $x0 $y1 $z0
				plane0 SetPoint2 $x0 $y0 $z1
		
				#back side
				plane1 SetOrigin $x0 $y0 $z0
				plane1 SetPoint1 $x0 $y1 $z0
				plane1 SetPoint2 $x1 $y0 $z0				

				#bottom side
				plane2 SetOrigin $x0 $y0 $z0
				plane2 SetPoint1 $x0 $y0 $z1
				plane2 SetPoint2 $x1 $y0 $z0

				#right side
				plane3 SetOrigin $x1 $y1 $z1
				plane3 SetPoint1 $x1 $y1 $z0
				plane3 SetPoint2 $x1 $y0 $z1

				#top side
				plane4 SetOrigin $x1 $y1 $z1
				plane4 SetPoint1 $x0 $y1 $z1
				plane4 SetPoint2 $x1 $y1 $z0

				#front side
				plane5 SetOrigin $x1 $y1 $z1
				plane5 SetPoint1 $x0 $y1 $z1
				plane5 SetPoint2 $x1 $y1 $z0			

				for {set i 0} {$i < 6} {incr i} {
					plane$i SetResolution 5 5
					stream$i SetSource [plane$i GetOutput]
					ren1 AddActor dataActor$i
					ren1 AddActor glyphActor$i
					glyph$i SetScaleFactor 5
				}

				

				ren1 AddActor bar
				ren1 AddActor outlineActor
				renWin Render
	} else { setData}
}
